import psutil
from bsb import Simulation, config, types

from .cell import ArborCell
from .connection import ArborConnection
from .device import ArborDevice


@config.node
class ArborSimulation(Simulation):
    """
    Interface between the scaffold model and the Arbor simulator.
    """

    resolution = config.attr(type=types.float(min=0.0), default=0.1)
    """Simulation time step size in milliseconds."""
    profiling = config.attr(type=bool)
    """Flag to perform profiling during the simulation."""
    cell_models: config._attrs.cfgdict[ArborCell] = config.dict(
        type=ArborCell, required=True
    )
    """Dictionary of cell models in the simulation."""
    connection_models: config._attrs.cfgdict[ArborConnection] = config.dict(
        type=ArborConnection, required=True
    )
    """Dictionary of connection models in the simulation."""
    devices: config._attrs.cfgdict[ArborDevice] = config.dict(
        type=ArborDevice, required=True
    )
    """Dictionary of devices in the simulation."""

    @config.property(default=1)
    def threads(self):
        return self._threads

    @threads.setter
    def threads(self, value):
        self._threads = value if value != "all" else psutil.cpu_count(logical=False)



import arbor
from arbor import units as U
from bsb import config

from ..connection import Receiver
from ..device import ArborDevice


@config.node
class PoissonGenerator(ArborDevice, classmap_entry="poisson_generator"):
    record = config.attr(type=bool, default=True)
    """Flag to save the spikes generated to file."""
    rate = config.attr(type=float, required=True)
    """Frequency of the poisson generator."""
    weight = config.attr(type=float, required=True)
    """Weight of the connection between the device and its target."""
    delay = config.attr(type=float, required=True)
    """Delay of the transmission between the device and its target."""

    def implement_probes(self, simdata, gid):
        return []

    def implement_generators(self, simdata, gid):
        target = Receiver(self, None, [-1, -1], [-1, -1], 0).on()
        gen = arbor.event_generator(
            target,
            self.weight,
            arbor.poisson_schedule(tstart=0 * U.ms, freq=self.rate * U.Hz, seed=gid),
        )
        return [gen]



"""
Arbor simulation adapter for the BSB framework.
"""

from bsb import SimulationBackendPlugin

from .adapter import ArborAdapter
from .devices import PoissonGenerator, Probe, SpikeRecorder
from .simulation import ArborSimulation

__plugin__ = SimulationBackendPlugin(Simulation=ArborSimulation, Adapter=ArborAdapter)


__all__ = [
    "PoissonGenerator",
    "Probe",
    "SpikeRecorder",
    "ArborAdapter",
    "ArborSimulation",
]
